function OnInit() //run upon initialization	
	brained = this:Data():GetNumWithDefault("brained", 0);
	
	if brained > 0 then
		color = this:GetBaseColor();
		this:SetBaseColor(Color(0, 0, 0, 0));
	end
	
	this:Data():SetIfNull("text", "Button");
	this:Data():SetIfNull("color", "0 0 255 200");
	this:Data():SetIfNull("scale", "1.0 1.0");
	this:Data():SetIfNull("alignment", tostring(C_ORIGIN_TOP_CENTER));
	this:Data():SetIfNull("offset", VectorToString(VECTOR2_ZERO));

	this:SetFont(this:Data():GetWithDefault("font", "font_custom_dialog"));
	this:SetText(this:Data():Get("text"));
	this:SetTextColor(StringToColor(this:Data():Get("color")));
	this:SetTextScale(StringToVector(this:Data():Get("scale")));
  
	this:SetTextAlignment(this:Data():GetNum("alignment"), StringToVector(this:Data():Get("offset")));
	this:SetRunUpdateEveryFrame(true);
	this:SetPriorityLevel(100);
end

function ScaleEntityToMatchText()
	
	local vecSize = this:GetTextBounds(); //see how big this text is
	  
	  //match it with our size, assuming our entity pic is 1,1 in size, this will scale
	  //correctly
	  
	  if (vecSize:Length() > 2) then
	  	this:SetScale(vecSize);
	  else
	  //well, the text is so tiny (or blank) I'd rather not scale it, might be hard to click
	  end
  
end

function SetText(text)
	
	this:Data():Set("text", tostring(text)); //store it for serializing to disk
    this:SetText(tostring(text));
 
  	ScaleEntityToMatchText();
  	
end

function SetTextScale(v)
	this:SetTextScale(v);
	
	//make this change persistant if it's moved/saved
	 this:Data():Set("scale", VectorToString(v));

	//ScaleEntityToMatchText();
end

function OnPostInit() //run during the entity's first logic update
	GetInputManager:AddBinding("return,always", "OnButton", this:GetID());
	GetInputManager:AddBinding("space,always", "OnButton", this:GetID());
	GetInputManager:AddBinding("mouse_left,always", "OnButton", this:GetID());
	GetInputManager:AddBinding("z,always", "OnButton", this:GetID());
	
	x1 = this:GetPos().x-this:GetSizeX()/2;
	y1 = this:GetPos().y-this:GetSizeY()/2;
	x2 = x1 + this:GetSizeX();
	y2 = y1 + this:GetSizeY();
	
	state = "base";
	btnPressed = false;
	//selectionMenu = "GameSelectMenu";
	selectionMenu = "LevelThemeMatter"; //disabled select menu for now
	defaultLevel = "Level0001";
	map = defaultLevel; //this extraneous var exists, to facilitate when code that adds more levels get to exist...
	btnType = this:Data():Get("btn_type");
	
	if btnType == "spw_manual" or btnType == "spw_automatic" then
		if wdata:GetNum("spawn_method") == 0 then
			this:Data():Set("btn_type", "spw_automatic");
		else
			this:Data():Set("btn_type", "spw_manual");
		end
		btnType = this:Data():Get("btn_type");
	end
	
	if btnType == "block_horz" or btnType == "free_horz" then
		if wdata:GetNum("allow_angle_h") == 0 then
			this:Data():Set("btn_type", "block_horz");
		else
			this:Data():Set("btn_type", "free_horz");
		end
		btnType = this:Data():Get("btn_type");
	end
	
	symbolBased = this:Data():GetNumWithDefault("symbol_based", 0);
	
	if brained > 0 then
		this:SetBaseColor(color);
		this:GetBrainManager():Add("FadeIn", "fade_speed_ms=" .. this:Data():GetNum("fade_speed"));
	end
end

function Update()
	if brained > 0 then
		if GetGameLogic:Data():GetNumWithDefault("logo_positioned", 0) == 1 then
			this:GetBrainManager():Remove("FadeIn");
			this:GetBrainManager():Add("FadeIn", "fade_speed_ms=" .. this:Data():GetNum("fade_speed"));
			brained = 0;
		else
			this:GetBrainManager():Add("FadeIn", "fade_speed_ms=" .. this:Data():GetNum("fade_speed"));
		end
	end

	if btnPressed == false and state != "gray" then
		if MouseHere() then
			if state == "base" then
				SoundSystem:Play(typebleep);
			end
			state = "roll";
			if btnType == "spw_manual" or btnType == "spw_automatic" then
				bcmethod_state = "roll";
			elseif btnType == "block_horz" or btnType == "free_horz" then
				blockHorz_state = "roll";
			end
		else
			state = "base";
			if btnType == "spw_manual" or btnType == "spw_automatic" then
				_G.bcmethod_state = "base";
			elseif btnType == "block_horz" or btnType == "free_horz" then
				_G.blockHorz_state = "base";
			end
		end
	end
	
	this:SetText(button_strings[btnType][g_language]);
	
	this:SetImage("/maps/_MenuArt/btn_base.png", nil);
	
	if this:GetBrainManager():GetBrainByName("FadeIn") == nil and g_editorActive == false then
		if state == "roll" then
			GetDrawManager:DrawImage("btn_roll.png", Vector2(x1, y1), Color(255, 255, 255, 255), C_LAYER_TOP, C_DRAW_WORLD_COORDS);
		else
			GetDrawManager:DrawImage("btn_cover.png", Vector2(x1, y1), Color(255, 255, 255, 255), C_LAYER_TOP, C_DRAW_WORLD_COORDS);
		end
	end
	
end

function OnButton(bKeyDown)
	if this:GetMap():GetName() != GetMapManager:GetActiveMap():GetName() then
		return true;
	end

	if bKeyDown then
		btnPressed = true;
	else
		btnPressed = false;
		if MouseHere() and state == "roll" then
			ButtonClicked();
			//return false;
		end
	end
	
	return true;
end

function ButtonClicked()

	//main menu
	if btnType == "new" then
		SelectionMenu();
	elseif btnType == "resume" then
		LoadGame();
	elseif btnType == "options" then
		GetMapManager:SetActiveMapByName("Options");
	elseif btnType == "credits" then
		GetMapManager:SetActiveMapByName("Credits");
	elseif btnType == "extras" then
		GetMapManager:SetActiveMapByName("Extras");
	elseif btnType == "quit" then
		GetGameLogic:Quit();
	elseif btnType == "playbtn" then
		StartGame();
	elseif btnType == "resetbtn" then
		GetGameLogic:SetRestartEngineFlag(true);
	elseif btnType == "power" then
		if PowerTransfer() then
			Unsummon_Dialog();
		end
	elseif btnType == "retry" then
		Unsummon_Dialog();
		ScheduleSystem(1, C_ENTITY_NONE, "LoadGame()");
	elseif btnType == "over" then
		Unsummon_Dialog();
		ClearSave();
		GetMapManager:SetActiveMapByName("High_Score");
		//summon gameover
	elseif btnType == "fullscreen" then
		if GetApp:GetIsWindowed() == true then
			GetApp:SetIsWindowed(false);
		else
			GetApp:SetIsWindowed(true);
		end
	elseif btnType == "spw_automatic" then
		wdata:SetNum("spawn_method", 1);
		this:Data():Set("btn_type", "spw_manual");
		btnType = this:Data():Get("btn_type");
	elseif btnType == "spw_manual" then
		wdata:SetNum("spawn_method", 0);
		this:Data():Set("btn_type", "spw_automatic");
		btnType = this:Data():Get("btn_type");
	elseif btnType == "block_horz" then
		wdata:SetNum("allow_angle_h", 1);
		this:Data():Set("btn_type", "free_horz");
		btnType = this:Data():Get("btn_type");
	elseif btnType == "free_horz" then
		wdata:SetNum("allow_angle_h", 0);
		this:Data():Set("btn_type", "block_horz");
		btnType = this:Data():Get("btn_type");
	end
end

function PowerTransfer()
	local matter = power_amount[C_MATTER];
	local energy = power_amount[C_ENERGY];
	local space = power_amount[C_SPACE];
	local var_time = power_amount[C_TIME];
	local sumPower = (matter + energy + space + var_time)/4;
	
	if sumPower <= 0 then
		return false;
	end
	
	Spawn_Change(sumPower ,C_SPWN_BAR_BIG);
	Spawn_Change(-matter ,C_SPWN_BAR_MATTER);
	Spawn_Change(-var_time ,C_SPWN_BAR_TIME);
	Spawn_Change(-energy ,C_SPWN_BAR_ENERGY);
	Spawn_Change(-space ,C_SPWN_BAR_SPACE);
	
	return true;
end

function SelectionMenu()
	//play selectionmenu music.
	GetMapManager:SetActiveMapByName(selectionMenu); 
	GetCamera:Reset(); 
end

function StartGame()
	g_SmartSound:FadeOutCurrentMusic(); //face out music, because the level selects its own music instead.
    GetMapManager:SetActiveMapByName(map); 
	GetCamera:Reset(); 
	Scores:Reset();
end


function OnKill() //run when removed
	
end

function MouseHere()
	mousePos = ScreenToWorld(GetInputManager:GetMousePos());

	if (mousePos.x > x1+20 and mousePos.x < x2-20) and (mousePos.y > y1+20 and mousePos.y < y2-20) then
		return true;
	else
		return false;
	end
end